/*
 * =====================================================
 * (c) Rick van de Westelaken & Bram van der Vlist 2007
 * MasterClass Technology fall/winter 2007
 * =====================================================
 * GUINNManager creates a Graphical User Interface which allows easy control of recording 
 * audio samples from a SoundSensor connected to a LEGO NXT brick, setting up a suitable 
 * Neural Network, training the network using Back-propagation algorithm and exporting 
 * the trained network to the NXT Brick.     
 */

import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.JFrame;
import javax.swing.UIManager;

public class GUINNManager extends javax.swing.JFrame {
    String conType;
    computerSideConnection csc = new computerSideConnection();	//make new CSC instance  
    Layer layer = new Layer();
	InitNN init = new InitNN();
	NNFileOutput fo = new NNFileOutput();
    
    /* Creates new form GUINNManager */
    public GUINNManager() {
        initComponents();
    }
    
    /**
     * Initializes the Graphical User Interface 
     */
    
    private void initComponents() {

        connectGroup = new javax.swing.ButtonGroup();
        panelConnection = new javax.swing.JPanel();
        radioBluetooth = new javax.swing.JRadioButton();
        radioUsb = new javax.swing.JRadioButton();
        buttonConnect = new javax.swing.JButton();
        lblConnection = new javax.swing.JLabel();
        txtMacadres = new javax.swing.JTextField();
        lblMacadres = new javax.swing.JLabel();
        panelNN = new javax.swing.JPanel();
        lblLearningRate = new javax.swing.JLabel();
        lblMomentum = new javax.swing.JLabel();
        lblHiddenNeurons = new javax.swing.JLabel();
        lblIterations = new javax.swing.JLabel();
        txtHiddenNeurons = new javax.swing.JTextField();
        txtLearningRate = new javax.swing.JTextField();
        txtMomentum = new javax.swing.JTextField();
        txtIterations = new javax.swing.JTextField();
        buttonTrain = new javax.swing.JButton();
        lblCalculatedError = new javax.swing.JLabel();
        panelTable = new javax.swing.JPanel();
        buttonRecord = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        buttonDelete = new javax.swing.JButton();
        panelExport = new javax.swing.JPanel();
        buttonExport = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Neural Nerwork manager for NXT");
        setName("frameMain");

        panelConnection.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "NXT Connection", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 11), new java.awt.Color(0, 0, 0)));

        connectGroup.add(radioBluetooth);
        radioBluetooth.setToolTipText("Use Bluetooth to connect to NXT");
        radioBluetooth.setText("Bluetooth");
        radioBluetooth.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radioBluetoothActionPerformed(evt);
            }
        });

        connectGroup.add(radioUsb);
        radioUsb.setSelected(true);
        radioUsb.setText("USB");
        radioUsb.setToolTipText("Use USB to connect to NXT");
        radioUsb.setContentAreaFilled(false);
        radioUsb.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radioUsbActionPerformed(evt);
            }
        });

        buttonConnect.setText("Connect to NXT");
        buttonConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonConnectActionPerformed(evt);
            }
        });

        lblConnection.setForeground(new java.awt.Color(204, 0, 0));
        lblConnection.setText("Disconnected");

        txtMacadres.setText("00:16:53:05:4D:9A");
        txtMacadres.setEnabled(false);
        txtMacadres.setVisible(false);
        

        lblMacadres.setText("Mac address");
        lblMacadres.setEnabled(false);
        lblMacadres.setVisible(false);
        
        javax.swing.GroupLayout panelConnectionLayout = new javax.swing.GroupLayout(panelConnection);
        panelConnection.setLayout(panelConnectionLayout);
        panelConnectionLayout.setHorizontalGroup(
            panelConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelConnectionLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(panelConnectionLayout.createSequentialGroup()
                        .addComponent(radioUsb)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(radioBluetooth))
                    .addComponent(buttonConnect, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGroup(panelConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelConnectionLayout.createSequentialGroup()
                        .addGap(26, 26, 26)
                        .addComponent(lblConnection)
                        .addContainerGap(134, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelConnectionLayout.createSequentialGroup()
                        .addGap(24, 24, 24)
                        .addComponent(lblMacadres)
                        .addGap(16, 16, 16)
                        .addComponent(txtMacadres, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(9, 9, 9))))
        );
        panelConnectionLayout.setVerticalGroup(
            panelConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, panelConnectionLayout.createSequentialGroup()
                .addContainerGap(11, Short.MAX_VALUE)
                .addGroup(panelConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txtMacadres, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(lblMacadres)
                    .addComponent(radioUsb)
                    .addComponent(radioBluetooth))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(buttonConnect)
                    .addComponent(lblConnection))
                .addContainerGap())
        );

        panelNN.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "Neural Network", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 11), new java.awt.Color(0, 0, 0)));

        lblLearningRate.setText("Learning rate");

        lblMomentum.setText("Momentum");

        lblHiddenNeurons.setText("Hidden neurons");

        lblIterations.setText("Iterations");

        buttonTrain.setText("Train network");
        buttonTrain.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonTrainActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout panelNNLayout = new javax.swing.GroupLayout(panelNN);
        panelNN.setLayout(panelNNLayout);
        panelNNLayout.setHorizontalGroup(
            panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelNNLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(lblCalculatedError, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 126, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(buttonTrain, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, panelNNLayout.createSequentialGroup()
                            .addGroup(panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(lblHiddenNeurons)
                                .addComponent(lblLearningRate)
                                .addComponent(lblMomentum)
                                .addComponent(lblIterations))
                            .addGap(18, 18, 18)
                            .addGroup(panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(txtIterations, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(txtMomentum, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(txtLearningRate, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(txtHiddenNeurons, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        panelNNLayout.setVerticalGroup(
            panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelNNLayout.createSequentialGroup()
                .addGap(28, 28, 28)
                .addGroup(panelNNLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(panelNNLayout.createSequentialGroup()
                        .addComponent(lblHiddenNeurons)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(lblLearningRate)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(lblMomentum)
                        .addGap(15, 15, 15)
                        .addComponent(lblIterations))
                    .addGroup(panelNNLayout.createSequentialGroup()
                        .addComponent(txtHiddenNeurons, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(txtLearningRate, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(txtMomentum, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(txtIterations, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(18, 18, 18)
                .addComponent(buttonTrain)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(lblCalculatedError, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        panelTable.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "Samples", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 11), new java.awt.Color(0, 0, 0)));

        buttonRecord.setText("Record");
        buttonRecord.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonRecordActionPerformed(evt);
            }
        });
        
        data = new Object[100][3];
        String[] columnNames = {"Sample name", "Output", "Values"};
        
        tableData = new javax.swing.JTable(data,columnNames);
       
        tableData.setModel(new javax.swing.table.DefaultTableModel(data, columnNames){
            boolean[] canEdit = new boolean [] {
                true, true, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        
        tableData.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        tableData.setColumnSelectionAllowed(true);
        tableData.setFillsViewportHeight(true);
        tableData.getTableHeader().setReorderingAllowed(false);
        jScrollPane1.setViewportView(tableData);
        tableData.getColumnModel().getSelectionModel().setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tableData.getColumnModel().getColumn(0).setPreferredWidth(158);
        tableData.getColumnModel().getColumn(1).setPreferredWidth(159);
        tableData.getColumnModel().getColumn(2).setResizable(false);
        tableData.getColumnModel().getColumn(2).setPreferredWidth(1);

        buttonDelete.setText("Remove");
        buttonDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonDeleteActionPerformed(evt);
            }
        });
        
        buttonDelete.setEnabled(false);
        buttonDelete.setVisible(false);

        javax.swing.GroupLayout panelTableLayout = new javax.swing.GroupLayout(panelTable);
        panelTable.setLayout(panelTableLayout);
        panelTableLayout.setHorizontalGroup(
            panelTableLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelTableLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelTableLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(panelTableLayout.createSequentialGroup()
                        .addComponent(buttonRecord)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(buttonDelete))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 338, Short.MAX_VALUE))
                .addContainerGap())
        );
        panelTableLayout.setVerticalGroup(
            panelTableLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelTableLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(panelTableLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(buttonDelete)
                    .addComponent(buttonRecord))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        panelExport.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "Export to NXT", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 11), new java.awt.Color(0, 0, 0)));

        buttonExport.setText("Export");
        buttonExport.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonExportActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout panelExportLayout = new javax.swing.GroupLayout(panelExport);
        panelExport.setLayout(panelExportLayout);
        panelExportLayout.setHorizontalGroup(
            panelExportLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelExportLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(buttonExport, javax.swing.GroupLayout.DEFAULT_SIZE, 128, Short.MAX_VALUE)
                .addContainerGap())
        );
        panelExportLayout.setVerticalGroup(
            panelExportLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelExportLayout.createSequentialGroup()
                .addGap(21, 21, 21)
                .addComponent(buttonExport)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(panelConnection, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(panelTable, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(panelExport, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(panelNN, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(10, 10, 10))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(panelNN, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(panelExport, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(panelConnection, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(panelTable, 0, 212, Short.MAX_VALUE)))
                .addContainerGap())
        );

        pack();
    }

    /**
     * ActionListeners that listen to button events and perform the desired actions. 
     */
    
    /*
     * Function creates a connection to the NXT brick using an USB or Bluetooth connection.
     * changes the text of label and button components of the GUI,
     * and makes the connection to the NXT according to current conType  
     */
    private void buttonConnectActionPerformed(java.awt.event.ActionEvent evt) {                                              
        if (connected == false)
        	{
        	buttonConnect.setText("Disconnect");
        	connected = true;
        	lblConnection.setText("Connected     ");
        	csc.makeConnection(txtMacadres.getText(), conType);
        	}
        else
        {
        	buttonConnect.setText("Connect to NXT");
        	connected = false;
        	lblConnection.setText("Disconnected");
        	csc.suicide();
        }
    }                                             

    /*
     * Function starts/stops recording and sets the values for the recorded data in the table
     * sample + sample number in column 0
     * 0 in column 1
     */
    private void buttonRecordActionPerformed(java.awt.event.ActionEvent evt) {                                            
        if (recording == false)
    	{	
        	buttonRecord.setText("   Stop  ");
        	recording = true;
        	csc.StartRecording();
    	}
        else
        {
            buttonRecord.setText("Record");
        	recording = false;
        	csc.StopRecording();
        	tableData.setValueAt("Sample " + (csc.SampleNbr), csc.SampleNbr, 0);
        	tableData.setValueAt(0, csc.SampleNbr, 1);
        }
    }  
    
    /*
     * Function exports the weights to a java file containing the neural network 
     */
    private void buttonExportActionPerformed(java.awt.event.ActionEvent evt) {                                             
    	fo.inSize=init.inputLayerSize;
    	fo.hidSize=init.hiddenLayerSize;
    	fo.outSize=init.outputLayerSize;
    	fo.setHiddenWeights(init.getHiddenWeights());
    	fo.setOutputWeights(init.getOutputWeights());
    	fo.exportClass();
    }                                            
    
    /*
     * Function that trains the network when train NN button is pressed
     */
    private void buttonTrainActionPerformed(java.awt.event.ActionEvent evt) {                                            
    	int hiddenNeurons;
    	int iterations;
    	double learningRate;
    	double momentum;
    	
    	// sets the default values for Learning rate, Momentum, number of hidden neurons and number of iterations  
        if (txtHiddenNeurons.getText().equals(""))
        {
        	String s = String.valueOf(50);
        	txtHiddenNeurons.setText(s);
        }
        if (txtIterations.getText().equals(""))
        {
        	String s1 = String.valueOf(1000);
           	txtIterations.setText(s1);
        }
        if (txtLearningRate.getText().equals(""))
        {
        	String s2 = String.valueOf(0.5);
            txtLearningRate.setText(s2);
        }
        if (txtMomentum.getText().equals(""))
        {
        	String s3 = String.valueOf(0.5);
         	txtMomentum.setText(s3);
        }
        
        //sets the values for training parameters with values from the GUI text fields
    	hiddenNeurons = Integer.parseInt(txtHiddenNeurons.getText());
		 
		iterations = Integer.parseInt(txtIterations.getText()); 
        
        learningRate = Double.parseDouble(txtLearningRate.getText());
		layer.updateLearningRate(learningRate);
        
        momentum = Double.parseDouble(txtMomentum.getText()); 
		layer.updateMomentum(momentum);
        
		// initializes the neural network, train the network and display's the network's accuracy  
		init.inputLayerSize = 50;
		init.hiddenLayerSize = Integer.parseInt(txtHiddenNeurons.getText());
		init.outputLayerSize = outputNeurons();
		init.setInputs(csc.getInputs());
		init.setOutputs(outputValues());
		init.createInputLayer();
		init.createHiddenLayer();
		init.createOutputLayer();
		init.createSynapses();
		init.backProp(iterations);

		lblCalculatedError.setText(String.valueOf(init.calcOutput()));
    }                                           
    
    /*
     * listener for the remove button (currently disabled and hidden)
     */
    private void buttonDeleteActionPerformed(java.awt.event.ActionEvent evt) {                                             
    	System.out.println(outputNeurons());
    	outputValues();
    }                                            
    
    /*
     * Radio button functions; set connection type for connecting and enabling mac-address input  
     */
    private void radioBluetoothActionPerformed(java.awt.event.ActionEvent evt) {
    	conType = "bt";
    	lblMacadres.setVisible(true);
    	txtMacadres.setVisible(true);
    	lblMacadres.setEnabled(true);
    	txtMacadres.setEnabled(true);
    }

    private void radioUsbActionPerformed(java.awt.event.ActionEvent evt) {
    	conType = "usb";
    	lblMacadres.setVisible(false);
    	txtMacadres.setVisible(false);
    	lblMacadres.setEnabled(false);
    	txtMacadres.setEnabled(false);
    }
    
    /*
     * Integer that returns the highest output number value (put in the table by user)
     * indicating the number of output neurons in the output layer  
     */
    public int outputNeurons() 
    {
 		int val=0;
 		for (int i=1;i<csc.SampleNbr + 1;i++)
 		{
 			if (Integer.parseInt(tableData.getValueAt(i, 1).toString()) > val){
 				val = Integer.parseInt(tableData.getValueAt(i, 1).toString());
 			}
 		}
		return (val +1);
    }
    
    /*
     * double returns a matrix containing the set output values for the network  
     */
    public double[][] outputValues(){
        double[][] outputMatrix = new double [csc.SampleNbr +1][outputNeurons()];
        
        for (int i=0; i<csc.SampleNbr +1; i++){
        	for (int j=0; j<outputNeurons(); j++){
        		outputMatrix[i][j] = 0;
        	}	
        }
        for (int i=0; i<csc.SampleNbr +1; i++){
        	outputMatrix[i][Integer.parseInt(tableData.getValueAt(i, 1).toString())] = 1.0;
        }
        
        return outputMatrix;
    }
    
    /*
     * centering the frame
     */
    public static void center(JFrame frame) {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        Point center = ge.getCenterPoint();
        Rectangle bounds = ge.getMaximumWindowBounds();
        int w = Math.max(bounds.width/2, Math.min(frame.getWidth(), bounds.width));
        int h = Math.max(bounds.height/2, Math.min(frame.getHeight(), bounds.height));
        int x = center.x - w/2, y = center.y - h/2;
        frame.setBounds(x, y, w, h);
    }
    
    /*
     * main function
     */    
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
            	UIManager.put("swing.boldMetal", Boolean.FALSE);
            	GUINNManager frame = new GUINNManager();
            	//center(frame);
            	frame.setSize(610, 390);
            	frame.setLocationRelativeTo(null);
            	frame.setVisible(true);
            }
        });
    }
    
    // Variables declaration
    private javax.swing.JButton buttonConnect;
    private javax.swing.JButton buttonDelete;
    private javax.swing.JButton buttonExport;
    private javax.swing.JButton buttonRecord;
    private javax.swing.JButton buttonTrain;
    private javax.swing.ButtonGroup connectGroup;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lblCalculatedError;
    private javax.swing.JLabel lblConnection;
    private javax.swing.JLabel lblHiddenNeurons;
    private javax.swing.JLabel lblIterations;
    private javax.swing.JLabel lblLearningRate;
    private javax.swing.JLabel lblMacadres;
    private javax.swing.JLabel lblMomentum;
    private javax.swing.JPanel panelConnection;
    private javax.swing.JPanel panelExport;
    private javax.swing.JPanel panelNN;
    private javax.swing.JPanel panelTable;
    private javax.swing.JRadioButton radioBluetooth;
    private javax.swing.JRadioButton radioUsb;
    private javax.swing.JTable tableData;
    public javax.swing.JTextField txtHiddenNeurons;
    public javax.swing.JTextField txtIterations;
    public javax.swing.JTextField txtLearningRate;
    public javax.swing.JTextField txtMacadres;
    public javax.swing.JTextField txtMomentum;
    private Object[][] data;
    public boolean connected = false;
    public boolean recording = false;
    // End of variables declaration
}
