/*
 * =====================================================
 * (c) Rick van de Westelaken & Bram van der Vlist 2007
 * MasterClass Technology fall/winter 2007
 * =====================================================
 */

import java.io.*;

public class NNFileOutput {
	int inSize;
	int hidSize;
	int outSize;
	double[][] hidWeights;
	double[][] outWeights;
	
	public void setHiddenWeights(double[][] weights){
		hidWeights = weights;
	}
	
	public void setOutputWeights(double[][] weights){
		outWeights = weights;
	}
	
	public void exportClass(){
		DataOutputStream fdos = null;
		File nnf = new File("nn.java");
		//check if file already exist. if so delete it!
		if (nnf.exists()){
			nnf.delete();
		}
		
		//Write data to nn.java
		String lineSeparator = System.getProperty("line.separator");
		try{
			fdos = new DataOutputStream(new FileOutputStream(nnf));
			
			fdos.writeBytes("//Autogenerated Neural Network");
			fdos.writeBytes(lineSeparator + "//by NXT neural network manager");
			fdos.writeBytes(lineSeparator + "//Made by Bram vd Vlist and Rick vd Westelaken (2007)");
			fdos.writeBytes(lineSeparator + "public class nn{");
			fdos.writeBytes(lineSeparator + "PreProcess pp = new PreProcess();");
			fdos.writeBytes(lineSeparator + "int inputSize = " + inSize + ";");
			fdos.writeBytes(lineSeparator + "int hiddenSize = " + hidSize + ";");
			fdos.writeBytes(lineSeparator + "int outputSize = " + outSize + ";");
			fdos.writeBytes(lineSeparator + "double[] hiddenValues = new double[hiddenSize];");
			fdos.writeBytes(lineSeparator + "double[] outputValues = new double[outputSize];");
			fdos.writeBytes(lineSeparator + "double[] inputValues = new double[inputSize];");

			//Print Hidden Weights
			fdos.writeBytes(lineSeparator + "double[][] hiddenWeights = new double[][]{");
			for (int i=0;i<hidWeights.length;i++){
				fdos.writeBytes(lineSeparator + "{");
				for (int j=0;j<hidWeights[0].length -1;j++){
					fdos.writeBytes(String.valueOf(hidWeights[i][j]));
					fdos.writeBytes(",");
				}
				fdos.writeBytes(String.valueOf(hidWeights[i][hidWeights[0].length-1]));
				fdos.writeBytes("}");
				if (i < hidWeights.length-1){
					fdos.writeBytes(",");	
				}
			}
			fdos.writeBytes(lineSeparator + "};");
			
			//Print Output weights
			fdos.writeBytes(lineSeparator + lineSeparator + "double[][] outputWeights = new double[][]{");
			for (int i=0;i<outWeights.length;i++){
				fdos.writeBytes(lineSeparator + "{");
				for (int j=0;j<outWeights[0].length -1;j++){
					fdos.writeBytes(String.valueOf(outWeights[i][j]));
					fdos.writeBytes(",");
				}
				fdos.writeBytes(String.valueOf(outWeights[i][outWeights[0].length-1]));
				fdos.writeBytes("}");
				if (i < outWeights.length-1){
					fdos.writeBytes(",");	
				}
			}
			fdos.writeBytes(lineSeparator + "};");
			
			fdos.writeBytes(lineSeparator + lineSeparator + "public void calcHiddenValues(){");
			fdos.writeBytes(lineSeparator + "  double sum=0;");
			fdos.writeBytes(lineSeparator + "  for (int h=0;h<hiddenSize;h++){");
			fdos.writeBytes(lineSeparator + "    sum=0;");
			fdos.writeBytes(lineSeparator + "    for (int i=0;i<inputSize;i++){");
			fdos.writeBytes(lineSeparator + "      sum += hiddenWeights[h][i] * inputValues[i];");
			fdos.writeBytes(lineSeparator + "    }");
			fdos.writeBytes(lineSeparator + "    hiddenValues[h] = (1.0 / (1.0 + Math.exp(sum)));");
			fdos.writeBytes(lineSeparator + "  }");
			fdos.writeBytes(lineSeparator + "}");
			
			fdos.writeBytes(lineSeparator + lineSeparator + "public void calcOutputValues(){");
			fdos.writeBytes(lineSeparator + "  double sum=0;");
			fdos.writeBytes(lineSeparator + "  for (int out=0;out<outputSize;out++){");
			fdos.writeBytes(lineSeparator + "    sum=0;");			
			fdos.writeBytes(lineSeparator + "    for (int h=0;h<hiddenSize;h++){");			
			fdos.writeBytes(lineSeparator + "      sum += hiddenWeights[out][h] * hiddenValues[h];");			
			fdos.writeBytes(lineSeparator + "    }");			
			fdos.writeBytes(lineSeparator + "    outputValues[out] = (1.0 / (1.0 + Math.exp(sum)));");			
			fdos.writeBytes(lineSeparator + "  }");			
			fdos.writeBytes(lineSeparator + "}");
			
			fdos.writeBytes(lineSeparator + "public int outputValue(){");			
			fdos.writeBytes(lineSeparator + "  int out = -1;");			
			fdos.writeBytes(lineSeparator + "  for (int i=0;i<outputSize;i++){");			
			fdos.writeBytes(lineSeparator + "    if (outputValues[i] > 0.5){");			
			fdos.writeBytes(lineSeparator + "      out=i;");			
			fdos.writeBytes(lineSeparator + "    }");			
			fdos.writeBytes(lineSeparator + "  }");			
			fdos.writeBytes(lineSeparator + "  return out;");			
			fdos.writeBytes(lineSeparator + "}");					
			
			fdos.writeBytes(lineSeparator + "public int getVoiceCommand(int[] rawInput){");			
			fdos.writeBytes(lineSeparator + "  inputValues = pp.calcNewArray(rawInput);");			
			fdos.writeBytes(lineSeparator + "  calcHiddenValues();");			
			fdos.writeBytes(lineSeparator + "  calcOutputValues();");			
			fdos.writeBytes(lineSeparator + "  return outputValue();");			
			fdos.writeBytes(lineSeparator + "}");			
			fdos.writeBytes(lineSeparator + lineSeparator + "}");				
		}
		catch(IOException ioe){
			
		}
		finally{
			try{
				//Close file
				if (fdos != null){
					fdos.close();
				}
			}
			catch (IOException ioe){
				
			}
		}
	}

}
