/*
 * =====================================================
 * (c) Rick van de Westelaken & Bram van der Vlist 2007
 * MasterClass Technology fall/winter 2007
 * =====================================================
 */

class Neuron
{
	public double momentum = 0.9;		// speeding convergence and avoiding local minima (0.0 - 1.0)
	public double learningRate = 0.05;	// defines the size of the steps for updating weights (0.0 - 1.0)
	double output; // range from 0.0 to 1.0
	double sum;
	double delta;
	Synapse[] inlinks;
	Synapse[] outlinks;
	int vNextS;
	int vPrevS;
  
	public Neuron(int prevs, int nexts)	// prevs = number of neurons in previous layer
	{									// nexts = number of neurons in next layer
		output   = 0.0;
		delta    = 0.0;
		sum      = 0.0;
		outlinks = new Synapse[nexts];
		inlinks  = new Synapse[prevs];
		vNextS=nexts;
		vPrevS=prevs;
	}
  
	public double getOutput()
	{
		return output;
	}
  
	public double getDelta()
	{
		return delta;
	}
  
	/*
	 * for all neurons in previous layer, get values and multiply by weight from incoming synapse 
	 */
	public void computeOutput()
	{
		sum=0.0;
		for (int i = 0; i < vPrevS; i++)
		{
			sum += inlinks[i].from.getOutput()*inlinks[i].getWeight();
		}
		output = 1.0/(1.0 + Math.exp(-sum)); // sigmoid function
	}
  
	/*
	 * for every neuron in the OUTPUT layer, calculate error signal (delta) 
	 */
	public void computeBackpropDelta(double d) // d = desired output
	{
		delta = (d - output) * output * (1.0 - output);
	}
  
	/*
	 * for every neuron in the HIDDEN layer, calculate error signal (delta)
	 */
	public void computeBackpropDelta() // for a hidden neuron
	{
		double errorSum = 0.0;
		for (int i = 0; i < vNextS; i++) 
		{
			errorSum += outlinks[i].to.delta * outlinks[i].getWeight();
		}
		delta = output * ( 1.0 - output) * errorSum;
	}
  
	/*
	 * update (incoming) weights, new weight = current weight + learning rate * delta 
	 * (computeBackpropDelta) * output previous neuron + momentum * delta 
	 */
	public void computeWeight()
	{
		for (int i = 0; i < vPrevS; i++)
		{
			inlinks[i].data = learningRate*delta*inlinks[i].from.getOutput()
                   + momentum*inlinks[i].data;
			inlinks[i].weight += inlinks[i].data;
		}
	}
	
	/*
	 * double that returns the weights for the all the incoming synapses of the neuron
	 */
	public double[] getWeights(){
		double[] nWeights = new double[vPrevS];
		for (int i = 0; i < vPrevS; i++)
		{
			nWeights[i] = inlinks[i].getWeight();
		}		
		return nWeights;
	}
}